/* BdfServer - Copyright (c) 2009-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.balayage;

import fr.exemole.bdfserver.api.balayage.Balayage;
import fr.exemole.bdfserver.api.balayage.BalayageConstants;
import fr.exemole.bdfserver.api.balayage.BalayageMode;
import fr.exemole.bdfserver.api.balayage.BalayageUnit;
import fr.exemole.bdfserver.api.balayage.Output;
import java.text.ParseException;
import java.util.AbstractList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.RandomAccess;
import java.util.Set;
import java.util.function.Predicate;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.extraction.def.AddendaExtractDef;
import net.fichotheque.extraction.def.AlbumExtractDef;
import net.fichotheque.extraction.def.CorpusExtractDef;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.extraction.def.ThesaurusExtractDef;
import net.fichotheque.extraction.filterunit.FilterUnit;
import net.fichotheque.tools.extraction.builders.AddendaExtractDefBuilder;
import net.fichotheque.tools.extraction.builders.AlbumExtractDefBuilder;
import net.fichotheque.tools.extraction.builders.CorpusExtractDefBuilder;
import net.fichotheque.tools.extraction.builders.ExtractionDefBuilder;
import net.fichotheque.tools.extraction.builders.FicheFilterBuilder;
import net.fichotheque.tools.extraction.builders.FilterUnitListBuilder;
import net.fichotheque.tools.extraction.builders.MotcleFilterBuilder;
import net.fichotheque.tools.extraction.builders.ThesaurusExtractDefBuilder;
import net.fichotheque.tools.extraction.builders.TitleClauseBuilder;
import net.fichotheque.utils.ExtractionUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.text.AccoladePattern;


/**
 *
 * @author Vincent Calame
 */
public final class BalayageUtils {

    public final static List<BalayageUnit> EMPTY_UNITLIST = Collections.emptyList();
    public final static List<BalayageMode> EMPTY_MODELIST = Collections.emptyList();
    public final static AccoladePattern UNIQUE_PATTERN;
    public final static AccoladePattern CORPUS_PATTERN;
    public final static AccoladePattern FICHE_PATTERN;
    public final static AccoladePattern THESAURUS_PATTERN;
    public final static AccoladePattern MOTCLE_PATTERN;
    public final static AccoladePattern SPHERE_PATTERN;
    public final static AccoladePattern REDACTEUR_PATTERN;
    public final static AccoladePattern DOCUMENT_PATTERN;
    public final static AccoladePattern ILLUSTRATION_PATTERN;
    private final static Predicate<String> ACCEPT_ALL_PREDICATE = new AcceptAllExtensionPredicate();
    private final static ExtractionDef CORPUS_DEFAULT_EXTRACTIONDEF = ExtractionDefBuilder.init()
            .setDynamicCorpusExtractDef(CorpusExtractDefBuilder
                    .init(ExtractionUtils.TITRE_FICHEFILTER)
                    .toCorpusExtractDef()
            )
            .toExtractionDef();
    private final static ExtractionDef FICHE_DEFAULT_EXTRACTIONDEF = ExtractionDefBuilder.init()
            .setDynamicCorpusExtractDef(CorpusExtractDefBuilder
                    .init(newFicheInstance())
                    .setClause(CorpusExtractDef.TITLE_CLAUSE, TitleClauseBuilder.DEFAULT)
                    .toCorpusExtractDef()
            )
            .toExtractionDef();
    private final static ExtractionDef MOTCLE_DEFAULT_EXTRACTIONDEF = ExtractionDefBuilder.init()
            .setDynamicThesaurusExtractDef(ThesaurusExtractDefBuilder
                    .init(MotcleFilterBuilder.init(MotcleFilter.DEFAULT_TYPE)
                            .setWithLabels(true)
                            .addDefaultIndexationCorpusExtractDef()
                            .toMotcleFilter())
                    .toThesaurusExtractDef()
            )
            .toExtractionDef();
    private final static ExtractionDef THESAURUS_DEFAULT_EXTRACTIONDEF = ExtractionDefBuilder.init()
            .setDynamicThesaurusExtractDef(ThesaurusExtractDefBuilder
                    .init(MotcleFilterBuilder.init(MotcleFilter.DEFAULT_TYPE)
                            .setWithLabels(true)
                            .addDefaultIndexationCorpusExtractDef()
                            .setChildrenRecursive()
                            .toMotcleFilter())
                    .setBoolean(ThesaurusExtractDef.WITHTHESAURUSTITLE_BOOLEAN, true)
                    .toThesaurusExtractDef()
            )
            .toExtractionDef();
    private final static ExtractionDef UNIQUE_DEFAULT_EXTRACTIONDEF = ExtractionDefBuilder.init().toExtractionDef();


    static {
        try {
            UNIQUE_PATTERN = new AccoladePattern("extraction");
            CORPUS_PATTERN = new AccoladePattern("corpus-{corpus}");
            FICHE_PATTERN = new AccoladePattern("fiche-{corpus}-{id}");
            THESAURUS_PATTERN = new AccoladePattern("thesaurus-{thesaurus}");
            MOTCLE_PATTERN = new AccoladePattern("motcle-{thesaurus}-{id}");
            SPHERE_PATTERN = new AccoladePattern("sphere-{sphere}");
            REDACTEUR_PATTERN = new AccoladePattern("redacteur-{sphere}-{id}");
            DOCUMENT_PATTERN = new AccoladePattern("{basename}");
            ILLUSTRATION_PATTERN = new AccoladePattern("{album}-{id}");
        } catch (ParseException pe) {
            throw new IllegalStateException(pe);
        }
    }

    private BalayageUtils() {
    }

    public static short langOptionToShort(String s) {
        if ((s == null) || (s.length() == 0)) {
            return BalayageConstants.LANG_OPTION_UNKNOWN;
        }
        if (s.equals("none")) {
            return BalayageConstants.LANG_OPTION_NONE;
        }
        if (s.equals("dir")) {
            return BalayageConstants.LANG_OPTION_DIR;
        }
        throw new IllegalArgumentException("unknown langOption : " + s);
    }

    /**
     *
     * @throws IllegalArgumentException si le type est incorrect.
     */
    public static String typeToString(short type) {
        switch (type) {
            case BalayageConstants.UNIQUE_TYPE:
                return "unique";
            case BalayageConstants.CORPUS_TYPE:
                return "corpus";
            case BalayageConstants.FICHE_TYPE:
                return "fiche";
            case BalayageConstants.THESAURUS_TYPE:
                return "thesaurus";
            case BalayageConstants.MOTCLE_TYPE:
                return "motcle";
            case BalayageConstants.SPHERE_TYPE:
                return "sphere";
            case BalayageConstants.REDACTEUR_TYPE:
                return "redacteur";
            case BalayageConstants.DOCUMENT_TYPE:
                return "document";
            case BalayageConstants.ILLUSTRATION_TYPE:
                return "illustration";
            default:
                throw new IllegalArgumentException("wrong type value: " + type);
        }
    }

    /**
     *
     * @throws IllegalArgumentException si le type est incorrect.
     */
    public static short typeToShort(String type) {
        if (type.equals("unique")) {
            return BalayageConstants.UNIQUE_TYPE;
        }
        if (type.equals("corpus")) {
            return BalayageConstants.CORPUS_TYPE;
        }
        if (type.equals("fiche")) {
            return BalayageConstants.FICHE_TYPE;
        }
        if (type.equals("thesaurus")) {
            return BalayageConstants.THESAURUS_TYPE;
        }
        if (type.equals("motcle")) {
            return BalayageConstants.MOTCLE_TYPE;
        }
        if (type.equals("sphere")) {
            return BalayageConstants.SPHERE_TYPE;
        }
        if (type.equals("redacteur")) {
            return BalayageConstants.REDACTEUR_TYPE;
        }
        if (type.equals("document")) {
            return BalayageConstants.DOCUMENT_TYPE;
        }
        if (type.equals("illustration")) {
            return BalayageConstants.ILLUSTRATION_TYPE;
        }
        throw new IllegalArgumentException("wrong type value");
    }

    /**
     *
     * @throws IllegalArgumentException si le type est incorrect.
     */
    public static AccoladePattern getDefaultPattern(short type) {
        switch (type) {
            case BalayageConstants.UNIQUE_TYPE:
                return UNIQUE_PATTERN;
            case BalayageConstants.CORPUS_TYPE:
                return CORPUS_PATTERN;
            case BalayageConstants.FICHE_TYPE:
                return FICHE_PATTERN;
            case BalayageConstants.THESAURUS_TYPE:
                return THESAURUS_PATTERN;
            case BalayageConstants.MOTCLE_TYPE:
                return MOTCLE_PATTERN;
            case BalayageConstants.SPHERE_TYPE:
                return SPHERE_PATTERN;
            case BalayageConstants.REDACTEUR_TYPE:
                return REDACTEUR_PATTERN;
            case BalayageConstants.DOCUMENT_TYPE:
                return DOCUMENT_PATTERN;
            case BalayageConstants.ILLUSTRATION_TYPE:
                return ILLUSTRATION_PATTERN;
            default:
                throw new SwitchException("wrong type = " + type);
        }
    }

    public static boolean containsMode(Balayage balayage, String modeName) {
        for (BalayageMode balayageMode : balayage.getBalayageModeList()) {
            if (balayageMode.getName().equals(modeName)) {
                return true;
            }
        }
        return false;
    }

    public static BalayageMode getMode(Balayage balayage, String modeName) {
        for (BalayageMode balayageMode : balayage.getBalayageModeList()) {
            if (balayageMode.getName().equals(modeName)) {
                return balayageMode;
            }
        }
        return null;
    }

    public static BalayageUnit getBalayageUnit(Balayage balayage, String unitName) {
        for (BalayageUnit balayageUnit : balayage.getBalayageUnitList()) {
            String bn = balayageUnit.getName();
            if (bn != null) {
                if (bn.equals(unitName)) {
                    return balayageUnit;
                }
            }
        }
        return null;
    }

    public static Output getOutput(BalayageUnit balayageUnit, String outputName) {
        int count = balayageUnit.getOutputCount();
        for (int i = 0; i < count; i++) {
            Output output = balayageUnit.getOutput(i);
            String on = output.getName();
            if (on != null) {
                if (on.equals(outputName)) {
                    return output;
                }
            }
        }
        return null;
    }

    public static AccoladePattern getPattern(Output output, short type) {
        AccoladePattern pattern = output.getPattern();
        if (pattern == null) {
            return getDefaultPattern(type);
        }
        return pattern;
    }

    public static Predicate<String> getExtensionPredicate(BalayageUnit balayageUnit) {
        List<String> extensionList = balayageUnit.getExtensionList();
        if (extensionList.isEmpty()) {
            return ACCEPT_ALL_PREDICATE;
        }
        return new SetExtensionPredicate(new HashSet<String>(extensionList));
    }


    public static ExtractionDef getDefaultExtractionDef(short balayageUnitType) {
        switch (balayageUnitType) {
            case BalayageConstants.CORPUS_TYPE:
                return CORPUS_DEFAULT_EXTRACTIONDEF;
            case BalayageConstants.FICHE_TYPE:
                return FICHE_DEFAULT_EXTRACTIONDEF;
            case BalayageConstants.MOTCLE_TYPE:
                return MOTCLE_DEFAULT_EXTRACTIONDEF;
            case BalayageConstants.THESAURUS_TYPE:
                return THESAURUS_DEFAULT_EXTRACTIONDEF;
            case BalayageConstants.UNIQUE_TYPE:
                return UNIQUE_DEFAULT_EXTRACTIONDEF;
            default:
                throw new SwitchException("wrong balayageUnitType = " + balayageUnitType);
        }
    }

    public static List<BalayageUnit> wrap(BalayageUnit[] array) {
        return new BalayageUnitList(array);
    }

    public static List<BalayageMode> wrap(BalayageMode[] array) {
        return new BalayageModeList(array);
    }

    public static FicheFilter newFicheInstance() {
        CorpusExtractDef corpusExtractDef = CorpusExtractDefBuilder
                .init(ExtractionUtils.TITRE_FICHEFILTER)
                .setClause(CorpusExtractDef.TITLE_CLAUSE, TitleClauseBuilder.DEFAULT)
                .toCorpusExtractDef();
        ThesaurusExtractDef thesaurusExtractDef = ThesaurusExtractDefBuilder
                .init(ExtractionUtils.LABELS_MOTCLEFILTER)
                .toThesaurusExtractDef();
        AlbumExtractDef albumExtractDef = AlbumExtractDefBuilder.init(ExtractionUtils.DEFAULT_ILLUSTRATIONFILTER).toAlbumExtractDef();
        AddendaExtractDef addendaExtractDef = AddendaExtractDefBuilder.init(ExtractionUtils.DEFAULT_DOCUMENTFILTER).toAddendaExtractDef();
        List<FilterUnit> filterUnitList = FilterUnitListBuilder.init()
                .addEnteteFilter()
                .addCorpsdeficheFilter()
                .addAddendaExtractFilter(addendaExtractDef)
                .addThesaurusExtractFilter(thesaurusExtractDef)
                .addCorpusExtractFilter(corpusExtractDef)
                .addAlbumExtractFilter(albumExtractDef)
                .addChronoFilter()
                .addFieldKeyFilter(FieldKey.LANG, null)
                .addPhraseFilter(FichothequeConstants.FICHE_PHRASE)
                .toFilterUnitList();
        return FicheFilterBuilder.init()
                .updateFilterUnitList(filterUnitList)
                .toFicheFilter();
    }


    private static class BalayageUnitList extends AbstractList<BalayageUnit> implements RandomAccess {

        private final BalayageUnit[] array;

        private BalayageUnitList(BalayageUnit[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public BalayageUnit get(int index) {
            return array[index];
        }

    }


    private static class BalayageModeList extends AbstractList<BalayageMode> implements RandomAccess {

        private final BalayageMode[] array;

        private BalayageModeList(BalayageMode[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public BalayageMode get(int index) {
            return array[index];
        }

    }


    private static class AcceptAllExtensionPredicate implements Predicate<String> {

        private AcceptAllExtensionPredicate() {
        }

        @Override
        public boolean test(String extension) {
            return true;
        }

    }


    private static class SetExtensionPredicate implements Predicate<String> {

        private final Set<String> set;

        private SetExtensionPredicate(Set<String> set) {
            this.set = set;
        }

        @Override
        public boolean test(String extension) {
            return set.contains(extension);
        }

    }

}
