/* BdfServer - Copyright (c) 2015-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ui.options;

import fr.exemole.bdfserver.api.ui.options.InputOption;
import fr.exemole.bdfserver.api.ui.options.LimitOption;
import fr.exemole.bdfserver.api.ui.options.ValueOption;
import java.util.function.Consumer;
import net.fichotheque.corpus.metadata.FieldKey;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.DomMessages;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public final class UiOptionsUtils {

    public final static LimitOption LIAGE_DEFAULT_LIMITOPTION = LimitOptionBuilder.init().toLimitOption();
    public final static ValueOption DEFAULT_VALUEOPTION = ValueOptionBuilder.init().toValueOption();
    public final static InputOption PROPRIETE_DEFAULT_INPUTOPTION = InputOptionBuilder.build(InputOption.INPUT_TEXT, InputOption.WIDTH_MEDIUM, 1);
    public final static InputOption SECTION_DEFAULT_INPUTOPTION = InputOptionBuilder.build(InputOption.INPUT_TEXT, InputOption.WIDTH_LARGE, 8);
    public final static InputOption INFORMATION_DEFAULT_INPUTOPTION = InputOptionBuilder.build(InputOption.INPUT_TEXT, InputOption.WIDTH_LARGE, 1);
    public final static InputOption TITRE_DEFAULT_INPUTOPTION = InputOptionBuilder.build(InputOption.INPUT_TEXT, InputOption.WIDTH_LARGE, 1);
    public final static InputOption REDACTEURS_DEFAULT_INPUTOPTION = InputOptionBuilder.build(InputOption.INPUT_TEXT, InputOption.WIDTH_SMALL, 1);
    public final static InputOption INDEXATION_DEFAULT_INPUTOPTION = InputOptionBuilder.build(InputOption.INPUT_TEXT, InputOption.WIDTH_LARGE, 1);
    public final static InputOption LANG_DEFAULT_INPUTOPTION = InputOptionBuilder.build(InputOption.INPUT_SELECT, InputOption.WIDTH_SMALL, 1);

    private UiOptionsUtils() {

    }

    /**
     * Peut-être nul
     */
    public static ValueOption getDefaultValueOption(FieldKey fieldKey) {
        switch (fieldKey.getFieldString()) {
            case FieldKey.SPECIAL_LANG:
            case FieldKey.SPECIAL_REDACTEURS:
            case FieldKey.SPECIAL_ID:
                return null;
            default:
                return DEFAULT_VALUEOPTION;
        }
    }

    public static InputOption getDefaultInputOption(FieldKey fieldKey) {
        switch (fieldKey.getCategory()) {
            case FieldKey.PROPRIETE_CATEGORY:
                return PROPRIETE_DEFAULT_INPUTOPTION;
            case FieldKey.INFORMATION_CATEGORY:
                return INFORMATION_DEFAULT_INPUTOPTION;
            case FieldKey.SECTION_CATEGORY:
                return SECTION_DEFAULT_INPUTOPTION;
            case FieldKey.SPECIAL_CATEGORY:
                switch (fieldKey.getFieldString()) {
                    case FieldKey.SPECIAL_TITRE:
                    case FieldKey.SPECIAL_SOUSTITRE:
                        return TITRE_DEFAULT_INPUTOPTION;
                    case FieldKey.SPECIAL_REDACTEURS:
                        return REDACTEURS_DEFAULT_INPUTOPTION;
                    case FieldKey.SPECIAL_LANG:
                        return LANG_DEFAULT_INPUTOPTION;
                    case FieldKey.SPECIAL_ID:
                        throw new IllegalArgumentException("No InputOption for FieldKey.ID");
                    default:
                        throw new SwitchException("Unknown special key = " + fieldKey.getFieldString());
                }
            default:
                throw new SwitchException("unknown category = " + fieldKey.getCategory());
        }
    }

    public final static String checkInputType(String inputType) {
        switch (inputType) {
            case InputOption.INPUT_TEXT:
            case InputOption.INPUT_SELECT:
            case InputOption.INPUT_CHECK:
            case InputOption.INPUT_RADIO:
            case InputOption.INPUT_NOTEDITABLE:
            case InputOption.INPUT_HIDDEN:
            case InputOption.INPUT_SUGGESTION:
            case InputOption.INPUT_FICHESTYLE:
                return inputType;
            default:
                throw new IllegalArgumentException("wrong value :" + inputType);
        }
    }

    public final static String checkWidthType(String widthType) {
        switch (widthType) {
            case InputOption.WIDTH_SMALL:
            case InputOption.WIDTH_MEDIUM:
            case InputOption.WIDTH_LARGE:
                return widthType;
            default:
                try {
                int size = Integer.parseInt(widthType);
                if (size < 10) {
                    return InputOption.WIDTH_SMALL;
                }
                if (size < 50) {
                    return InputOption.WIDTH_MEDIUM;
                }
                return InputOption.WIDTH_LARGE;
            } catch (NumberFormatException nfe) {
                throw new IllegalArgumentException("wrong value : " + widthType);
            }
        }
    }

    public static LimitOption readLimitOption(Element element, MessageHandler messageHandler) {
        String xpath = "limit";
        LimitOptionBuilder limitOptionBuilder = new LimitOptionBuilder();
        limitOptionBuilder.setNoNewAllowed(StringUtils.isTrue(element.getAttribute("nonew")));
        return limitOptionBuilder.toLimitOption();
    }

    public static ValueOption readValueOption(Element element, MessageHandler messageHandler) {
        String xpath = "value";
        ValueOptionBuilder valueOptionBuilder = new ValueOptionBuilder();
        valueOptionBuilder.setMandatory(StringUtils.isTrue(element.getAttribute("mandatory")));
        String defaultValue = element.getAttribute("default");
        if (defaultValue.length() != 0) {
            valueOptionBuilder.setDefaultValue(defaultValue);
        }
        DOMUtils.readChildren(element, new ValueConsumer(valueOptionBuilder, messageHandler));
        return valueOptionBuilder.toValueOption();
    }

    public static InputOption readInputOption(Element element, MessageHandler messageHandler) {
        String xpath = "input";
        InputOptionBuilder inputOptionBuilder = new InputOptionBuilder();
        String inputType = element.getAttribute("type");
        if (inputType.length() == 0) {
            DomMessages.emptyAttribute(messageHandler, xpath, "type");
            return null;
        }
        try {
            inputOptionBuilder.setInputType(inputType);
        } catch (IllegalArgumentException iae) {
            DomMessages.wrongAttributeValue(messageHandler, xpath, "type", inputType);
            return null;
        }
        String widthType = element.getAttribute("width");
        if (widthType.length() == 0) {
            widthType = element.getAttribute("size"); //Ancien nom de l'attribut
        }
        if (widthType.length() > 0) {
            try {
                inputOptionBuilder.setWidthType(widthType);
            } catch (IllegalArgumentException iae) {
                DomMessages.wrongAttributeValue(messageHandler, xpath, "width", widthType);
            }
        }
        String rowsString = element.getAttribute("rows");
        if (rowsString.length() > 0) {
            try {
                int row = Integer.parseInt(rowsString);
                inputOptionBuilder.setRows(row);
            } catch (NumberFormatException nfe) {
                DomMessages.wrongIntegerAttributeValue(messageHandler, xpath, "rows", rowsString);
            } catch (IllegalArgumentException iae) {
                DomMessages.wrongAttributeValue(messageHandler, xpath, "rows", rowsString);
            }
        }
        return inputOptionBuilder.toInputOption();
    }


    private static class ValueConsumer implements Consumer<Element> {

        private final ValueOptionBuilder valueOptionBuilder;
        private final MessageHandler messageHandler;


        private ValueConsumer(ValueOptionBuilder valueOptionBuilder, MessageHandler messageHandler) {
            this.valueOptionBuilder = valueOptionBuilder;
            this.messageHandler = messageHandler;
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("default")) {
                valueOptionBuilder.setDefaultValue(XMLUtils.getRawData(element));
            } else {
                DomMessages.unknownTagWarning(messageHandler, tagName);
            }
        }

    }


}
