/* BdfServer_Multi - Copyright (c) 2015-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.multi;

import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.multi.api.Multi;
import fr.exemole.bdfserver.multi.api.MultiConstants;
import fr.exemole.bdfserver.multi.commands.IniCommand;
import fr.exemole.bdfserver.multi.htmlproducers.AdminHtmlProducer;
import fr.exemole.bdfserver.multi.htmlproducers.AppHtmlProducer;
import fr.exemole.bdfserver.multi.htmlproducers.CrossLoginHtmlProducer;
import fr.exemole.bdfserver.multi.htmlproducers.IndexHtmlProducer;
import fr.exemole.bdfserver.multi.htmlproducers.IniFormHtmlProducer;
import fr.exemole.bdfserver.multi.htmlproducers.IniResultHtmlProducer;
import fr.exemole.bdfserver.multi.htmlproducers.MultiHtmlProducer;
import fr.exemole.bdfserver.multi.jsonproducers.MultiJsonProducer;
import fr.exemole.bdfserver.multi.jsonproducers.admin.AdminJsonProducerFactory;
import fr.exemole.bdfserver.multi.jsonproducers.user.UserJsonProducerFactory;
import fr.exemole.bdfserver.multi.streamproducers.SynthesisOdsProducer;
import fr.exemole.bdfserver.multi.tools.CrossInfo;
import fr.exemole.bdfserver.tools.apps.AppConf;
import java.text.ParseException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangPreference;
import net.mapeadores.util.localisation.LangPreferenceBuilder;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.RequestConstants;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.request.RequestUtils;
import net.mapeadores.util.servlets.HttpServletRequestMap;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.ServletUtils;
import net.mapeadores.util.servlets.handlers.CommandMessageResponseHandler;
import net.mapeadores.util.servlets.handlers.JsonResponseHandler;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public final class MultiResponseHandlerFactory {

    private MultiResponseHandlerFactory() {

    }

    public static ResponseHandler getResourceResponseHandler(Multi multi, String path) {
        RelativePath relativePath;
        try {
            relativePath = RelativePath.parse(path);
        } catch (ParseException pe) {
            return null;
        }
        DocStream docStream = multi.getWebappsResourceStorages().getResourceDocStream(relativePath);
        if (docStream == null) {
            return null;
        }
        return ServletUtils.wrap(docStream);
    }

    public static ResponseHandler getIniResponseHandler(Multi multi, RequestMap requestMap) {
        String cmd = requestMap.getParameter(RequestConstants.COMMAND_PARAMETER);
        IniCommand.Result result = null;
        if ((cmd != null) && (cmd.equals(IniCommand.COMMAND_NAME))) {
            IniCommand iniCommand = new IniCommand(requestMap, multi);
            result = iniCommand.doIniCommand();
        }
        MultiHtmlProducer htmlProducer;
        if ((result != null) && (!result.hasError())) {
            htmlProducer = new IniResultHtmlProducer(multi, result);
        } else {
            htmlProducer = new IniFormHtmlProducer(multi, requestMap);
        }
        if (result != null) {
            htmlProducer.setCommandMessage(result.getCommandMessage());
        }
        return ServletUtils.wrap(htmlProducer);
    }

    public static ResponseHandler getAppHtmlRespondeHandler(Multi multi, AppConf appConf, BdfUser bdfUser) {
        AppHtmlProducer htmlProducer;
        if (bdfUser != null) {
            htmlProducer = new AppHtmlProducer(multi, appConf, bdfUser);
        } else {
            htmlProducer = new AppHtmlProducer(multi, appConf);
        }
        return ServletUtils.wrap(htmlProducer);
    }

    public static ResponseHandler getIndexHtmlResponseHandler(Multi multi, CrossInfo crossInfo) {
        IndexHtmlProducer htmlProducer = new IndexHtmlProducer(multi, crossInfo);
        return ServletUtils.wrap(htmlProducer);
    }

    public static ResponseHandler getCrossLoginJsonResponseHandler(Multi multi, CrossInfo crossInfo, RequestMap requestMap, String output) {
        short jsonType = RequestUtils.getJsonType(requestMap);
        String callback = requestMap.getParameter(RequestConstants.CALLBACK_PARAMETER);
        MultiJsonProducer jsonProducer = new MultiJsonProducer(false, multi.getMessageLocalisationProvider().getMessageLocalisation(getLangPreference(multi, requestMap)));
        return new JsonResponseHandler(jsonProducer, jsonType, callback);
    }

    public static ResponseHandler getCrossLoginHtmlResponseHandler(Multi multi, CrossInfo crossInfo, RequestMap requestMap) {
        CrossLoginHtmlProducer htmlProducer = new CrossLoginHtmlProducer(multi, crossInfo, getLangPreference(multi, requestMap));
        return ServletUtils.wrap(htmlProducer);
    }

    public static ResponseHandler getAdminHtmlRespondeHandler(Multi multi) {
        AdminHtmlProducer htmlProducer = new AdminHtmlProducer(multi);
        return ServletUtils.wrap(htmlProducer);
    }

    public static ResponseHandler getAdminJsonRespondeHandler(Multi multi, short check, RequestMap requestMap, String output) {
        short jsonType = RequestUtils.getJsonType(requestMap);
        if (output == null) {
            output = MultiConstants.PING_JSON;
        }
        String callback = requestMap.getParameter(RequestConstants.CALLBACK_PARAMETER);
        MultiJsonProducer jsonProducer = null;
        switch (check) {
            case MultiConf.AUTHREQUIRED_STATE:
                jsonProducer = AdminJsonProducerFactory.unauthentified(multi);
                break;
            case MultiConf.AUTHFAILED_STATE:
                jsonProducer = AdminJsonProducerFactory.unauthentified(multi).setCommandMessage(LogUtils.error("_ warning.multi.authentification_failed"));
                break;
            case MultiConf.CONNECTED_STATE:
                CommandMessage commandMessage = MultiAdminCommands.run(multi, requestMap);
                jsonProducer = AdminJsonProducerFactory.getJsonProducer(multi, output, commandMessage, requestMap);
                break;
            default:
                throw new SwitchException("Unknown check: " + check);
        }
        return new JsonResponseHandler(jsonProducer, jsonType, callback);
    }

    public static ResponseHandler getAdminStreamRespondeHandler(Multi multi, String output) {
        switch (output) {
            case MultiConstants.SYNTHESIS_STREAM:
                if (multi.isAuthentificationSharingAllowed()) {
                    return ServletUtils.wrap(new SynthesisOdsProducer(multi, MultiUtils.getSynthesis(multi)));
                } else {
                    return CommandMessageResponseHandler.getHtmlInstance(multi.getAdminMessageLocalisation(), LogUtils.error("_ error.unsupported.sharingnotallowed"));
                }
            default:
                return CommandMessageResponseHandler.getHtmlInstance(multi.getAdminMessageLocalisation(), LogUtils.error("_ error.unknown.parametervalue", RequestConstants.STREAM_PARAMETER, output));
        }
    }

    public static ResponseHandler getUserJsonRespondeHandler(Multi multi, BdfUser bdfUser, HttpServletRequestMap requestMap, String output) {
        if (output == null) {
            output = "";
        }
        JsonProducer jsonProducer = UserJsonProducerFactory.getJsonProducer(multi, bdfUser, output);
        short jsonType = RequestUtils.getJsonType(requestMap);
        String callback = requestMap.getParameter(RequestConstants.CALLBACK_PARAMETER);
        return new JsonResponseHandler(jsonProducer, jsonType, callback);
    }

    public static LangPreference getLangPreference(Multi multi, RequestMap requestMap) {
        LangPreferenceBuilder langPreferenceBuilder = new LangPreferenceBuilder();
        RequestUtils.checkLangPreference(requestMap, langPreferenceBuilder);
        langPreferenceBuilder.addLangs(multi.getMultiMetadata().getWorkingLangs());
        langPreferenceBuilder.addLang(Lang.build("fr"));
        return langPreferenceBuilder.toLangPreference();
    }

}
