/* FichothequeLib_API - Copyright (c) 2011-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.utils;

import java.util.AbstractList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.RandomAccess;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.extraction.def.AddendaExtractDef;
import net.fichotheque.extraction.def.AlbumExtractDef;
import net.fichotheque.extraction.def.CorpusExtractDef;
import net.fichotheque.extraction.def.FicheRattachementDef;
import net.fichotheque.extraction.def.MotcleRattachementDef;
import net.fichotheque.extraction.def.TagNameInfo;
import net.fichotheque.extraction.def.ThesaurusExtractDef;
import net.fichotheque.extraction.filterunit.AddendaExtractFilterUnit;
import net.fichotheque.extraction.filterunit.AlbumExtractFilterUnit;
import net.fichotheque.extraction.filterunit.ChronoFilterUnit;
import net.fichotheque.extraction.filterunit.CorpsdeficheFilterUnit;
import net.fichotheque.extraction.filterunit.CorpusExtractFilterUnit;
import net.fichotheque.extraction.filterunit.EnteteFilterUnit;
import net.fichotheque.extraction.filterunit.FicheRattachementFilterUnit;
import net.fichotheque.extraction.filterunit.FieldKeyFilterUnit;
import net.fichotheque.extraction.filterunit.FieldNamePrefixFilterUnit;
import net.fichotheque.extraction.filterunit.FilterUnit;
import net.fichotheque.extraction.filterunit.LotFilterUnit;
import net.fichotheque.extraction.filterunit.MotcleRattachementFilterUnit;
import net.fichotheque.extraction.filterunit.PhraseFilterUnit;
import net.fichotheque.extraction.filterunit.ThesaurusExtractFilterUnit;
import net.mapeadores.util.attr.Attributes;


/**
 *
 * @author Vincent Calame
 */
public final class FilterUnitUtils {

    public final static EnteteFilterUnit ENTETE_FILTERUNIT = new InternalEnteteFilterUnit();
    public final static CorpsdeficheFilterUnit CORPSDEFICHE_FILTERUNIT = new InternalCorpsdeficheFilterUnit();
    public final static ChronoFilterUnit CHRONO_FILTERUNIT = new InternalChronoFilterUnit();
    public final static PhraseFilterUnit FICHEPHRASE_FILTERUNIT = new InternalPhraseFilterUnit(FichothequeConstants.FICHE_PHRASE);
    public final static List<FilterUnit> EMPTY_FILTERUNITLIST = Collections.emptyList();

    private FilterUnitUtils() {
    }

    public static FieldKeyFilterUnit toFieldKeyFilterUnit(FieldKey fieldKey, Attributes attributes) {
        return new InternalFieldKeyFilterUnit(fieldKey, attributes);
    }

    public static LotFilterUnit toLotFilterUnit(TagNameInfo tagNameInfo, List<FilterUnit> filterUnitList) {
        return new InternalLotFilterUnit(tagNameInfo, toImmutableList(filterUnitList));
    }

    public static FieldNamePrefixFilterUnit toFieldNamePrefixFilterUnit(short category, String prefix, Attributes attributes) {
        return new InternalFieldNamePrefixFilterUnit(category, prefix, attributes);
    }

    public static CorpusExtractFilterUnit toCorpusExtractFilterUnit(CorpusExtractDef corpusExtractDef) {
        return new InternalCorpusExtractFilterUnit(corpusExtractDef);
    }

    public static ThesaurusExtractFilterUnit toThesaurusExtractFilterUnit(ThesaurusExtractDef thesaurusExtractDef) {
        return new InternalThesaurusExtractFilterUnit(thesaurusExtractDef);
    }

    public static AlbumExtractFilterUnit toAlbumExtractFilterUnit(AlbumExtractDef albumExtractDef) {
        return new InternalAlbumExtractFilterUnit(albumExtractDef);
    }

    public static AddendaExtractFilterUnit toAddendaExtractFilterUnit(AddendaExtractDef addendaExtractDef) {
        return new InternalAddendaExtractFilterUnit(addendaExtractDef);
    }

    public static FicheRattachementFilterUnit toFicheRattachementFilterUnit(FicheRattachementDef ficheRattachementDef) {
        return new InternalFicheRattachementFilterUnit(ficheRattachementDef);
    }

    public static MotcleRattachementFilterUnit toMotcleRattachementFilterUnit(MotcleRattachementDef motcleRattachementDef) {
        return new InternalMotcleRattachementFilterUnit(motcleRattachementDef);
    }

    public static PhraseFilterUnit toPhraseFilterUnit(String phraseName) {
        switch (phraseName) {
            case FichothequeConstants.FICHE_PHRASE:
                return FICHEPHRASE_FILTERUNIT;
            default:
                return new InternalPhraseFilterUnit(phraseName);
        }
    }


    private static class InternalEnteteFilterUnit implements EnteteFilterUnit {

        private InternalEnteteFilterUnit() {
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalCorpsdeficheFilterUnit implements CorpsdeficheFilterUnit {

        private InternalCorpsdeficheFilterUnit() {
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalChronoFilterUnit implements ChronoFilterUnit {

        private InternalChronoFilterUnit() {
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalPhraseFilterUnit implements PhraseFilterUnit {

        private final String phraseName;

        private InternalPhraseFilterUnit(String phraseName) {
            this.phraseName = phraseName;
        }

        @Override
        public String getName() {
            return phraseName;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalFieldKeyFilterUnit implements FieldKeyFilterUnit {

        private final FieldKey fieldKey;
        private final Attributes attributes;

        private InternalFieldKeyFilterUnit(FieldKey fieldKey, Attributes attributes) {
            this.fieldKey = fieldKey;
            this.attributes = attributes;
        }

        @Override
        public FieldKey getFieldKey() {
            return fieldKey;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }


    private static class InternalLotFilterUnit implements LotFilterUnit {

        private final TagNameInfo tagNameInfo;
        private final List<FilterUnit> filterUnitList;

        private InternalLotFilterUnit(TagNameInfo tagNameInfo, List<FilterUnit> filterUnitList) {
            this.tagNameInfo = tagNameInfo;
            this.filterUnitList = filterUnitList;
        }

        @Override
        public TagNameInfo getTagNameInfo() {
            return tagNameInfo;
        }

        @Override
        public List<FilterUnit> getFilterUnitList() {
            return filterUnitList;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalFieldNamePrefixFilterUnit implements FieldNamePrefixFilterUnit {

        private final short category;
        private final String prefix;
        private final Attributes attributes;

        private InternalFieldNamePrefixFilterUnit(short category, String prefix, Attributes attributes) {
            this.category = category;
            this.prefix = prefix;
            this.attributes = attributes;
        }

        @Override
        public short getCategory() {
            return category;
        }

        @Override
        public String getPrefix() {
            return prefix;

        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }


    private static class InternalCorpusExtractFilterUnit implements CorpusExtractFilterUnit {

        private final CorpusExtractDef corpusExtractDef;

        private InternalCorpusExtractFilterUnit(CorpusExtractDef corpusExtractDef) {
            this.corpusExtractDef = corpusExtractDef;
        }

        @Override
        public CorpusExtractDef getCorpusExtractDef() {
            return corpusExtractDef;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalThesaurusExtractFilterUnit implements ThesaurusExtractFilterUnit {

        private final ThesaurusExtractDef thesaurusExtractDef;

        private InternalThesaurusExtractFilterUnit(ThesaurusExtractDef thesaurusExtractDef) {
            this.thesaurusExtractDef = thesaurusExtractDef;
        }

        @Override
        public ThesaurusExtractDef getThesaurusExtractDef() {
            return thesaurusExtractDef;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalAlbumExtractFilterUnit implements AlbumExtractFilterUnit {

        private final AlbumExtractDef albumExtractDef;

        private InternalAlbumExtractFilterUnit(AlbumExtractDef albumExtractDef) {
            this.albumExtractDef = albumExtractDef;
        }

        @Override
        public AlbumExtractDef getAlbumExtractDef() {
            return albumExtractDef;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalAddendaExtractFilterUnit implements AddendaExtractFilterUnit {

        private final AddendaExtractDef addendaExtractDef;

        private InternalAddendaExtractFilterUnit(AddendaExtractDef addendaExtractDef) {
            this.addendaExtractDef = addendaExtractDef;
        }

        @Override
        public AddendaExtractDef getAddendaExtractDef() {
            return addendaExtractDef;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalFicheRattachementFilterUnit implements FicheRattachementFilterUnit {

        private final FicheRattachementDef ficheRattachementDef;

        private InternalFicheRattachementFilterUnit(FicheRattachementDef ficheRattachementDef) {
            this.ficheRattachementDef = ficheRattachementDef;
        }

        @Override
        public FicheRattachementDef getFicheRattachementDef() {
            return ficheRattachementDef;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }


    private static class InternalMotcleRattachementFilterUnit implements MotcleRattachementFilterUnit {

        private final MotcleRattachementDef ficheRattachementDef;

        private InternalMotcleRattachementFilterUnit(MotcleRattachementDef ficheRattachementDef) {
            this.ficheRattachementDef = ficheRattachementDef;
        }

        @Override
        public MotcleRattachementDef getMotcleRattachementDef() {
            return ficheRattachementDef;
        }

        @Override
        public Attributes getAttributes() {
            return null;
        }

    }

    public static List<FilterUnit> toImmutableList(Collection<FilterUnit> collection) {
        return new FilterUnitList(collection.toArray(new FilterUnit[collection.size()]));
    }

    public static List<FilterUnit> wrap(FilterUnit[] array) {
        return new FilterUnitList(array);
    }


    private static class FilterUnitList extends AbstractList<FilterUnit> implements RandomAccess {

        private final FilterUnit[] array;

        private FilterUnitList(FilterUnit[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public FilterUnit get(int index) {
            return array[index];
        }

    }

}
