/* FichothequeLib_Tools - Copyright (c) 2014-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction;

import java.util.function.Predicate;
import net.fichotheque.Fichotheque;
import net.fichotheque.Subset;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.LinkAnalyser;
import net.fichotheque.extraction.run.ExtractorProvider;
import net.mapeadores.util.localisation.LangContext;
import net.mapeadores.util.localisation.MessageLocalisationProvider;
import net.mapeadores.util.mimetype.MimeTypeResolver;
import net.mapeadores.util.mimetype.MimeTypeUtils;
import net.fichotheque.corpus.Fiches;
import net.fichotheque.extraction.SyntaxResolver;


/**
 *
 * @author Vincent Calame
 */
public class ExtractionContextBuilder {

    private final Fichotheque fichotheque;
    private final MessageLocalisationProvider messageLocalisationProvider;
    private final LangContext langContext;
    private Predicate<Subset> subsetAccessPredicate;
    private Predicate<FicheMeta> fichePredicate = null;
    private Fiches selectedFiches = null;
    private ExtractorProvider extractorProvider = ExtractionContextUtils.DEFAULT_EXTRACTORPROVIDER;
    private LinkAnalyser linkAnalyser = ExtractionContextUtils.DEFAULT_LINKANALYSER;
    private MimeTypeResolver mimeTypeResolver = MimeTypeUtils.DEFAULT_RESOLVER;
    private SyntaxResolver modeResolver = ExtractionContextUtils.DEFAULT_SYNTAXRESOLVER;

    public ExtractionContextBuilder(Fichotheque fichotheque, MessageLocalisationProvider messageLocalisationProvider, LangContext langContext) {
        this.fichotheque = fichotheque;
        this.messageLocalisationProvider = messageLocalisationProvider;
        this.langContext = langContext;
    }

    public ExtractionContextBuilder setSubsetAccessPredicate(Predicate<Subset> subsetAccessPredicate) {
        this.subsetAccessPredicate = subsetAccessPredicate;
        return this;
    }

    public ExtractionContextBuilder setFichePredicate(Predicate<FicheMeta> fichePredicate) {
        this.fichePredicate = fichePredicate;
        return this;
    }

    public ExtractionContextBuilder setSelectedFiches(Fiches selectedFiches) {
        this.selectedFiches = selectedFiches;
        return this;
    }

    public ExtractionContextBuilder setExtractorProvider(ExtractorProvider extractorProvider) {
        if (extractorProvider == null) {
            this.extractorProvider = ExtractionContextUtils.DEFAULT_EXTRACTORPROVIDER;
        } else {
            this.extractorProvider = extractorProvider;
        }
        return this;
    }

    public ExtractionContextBuilder setLinkAnalyser(LinkAnalyser linkAnalyser) {
        if (linkAnalyser == null) {
            this.linkAnalyser = ExtractionContextUtils.DEFAULT_LINKANALYSER;
        } else {
            this.linkAnalyser = linkAnalyser;
        }
        return this;
    }

    public ExtractionContextBuilder setMimeTypeResolver(MimeTypeResolver mimeTypeResolver) {
        if (mimeTypeResolver == null) {
            this.mimeTypeResolver = MimeTypeUtils.DEFAULT_RESOLVER;
        } else {
            this.mimeTypeResolver = mimeTypeResolver;
        }
        return this;
    }

    public ExtractionContextBuilder setModeResolver(SyntaxResolver modeResolver) {
        if (modeResolver == null) {
            this.modeResolver = ExtractionContextUtils.DEFAULT_SYNTAXRESOLVER;
        } else {
            this.modeResolver = modeResolver;
        }
        return this;
    }

    public ExtractionContext toExtractionContext() {
        Predicate<Subset> finalSubsetAccessPredicate;
        if (subsetAccessPredicate != null) {
            finalSubsetAccessPredicate = subsetAccessPredicate;
        } else {
            throw new IllegalStateException("subsetAccessPredicate must be defined");
            //finalSubsetAccessPredicate = EligibilityUtils.ALL_SUBSET_PREDICATE;
        }
        return new InternalExtractionContext(fichotheque, messageLocalisationProvider, langContext, extractorProvider, finalSubsetAccessPredicate, fichePredicate, selectedFiches, linkAnalyser, mimeTypeResolver, modeResolver);
    }

    public static ExtractionContextBuilder init(Fichotheque fichotheque, MessageLocalisationProvider messageLocalisationProvider, LangContext langContext) {
        return new ExtractionContextBuilder(fichotheque, messageLocalisationProvider, langContext);
    }


    private static class InternalExtractionContext implements ExtractionContext {

        private final Fichotheque fichotheque;
        private final MessageLocalisationProvider messageLocalisationProvider;
        private final LangContext langContext;
        private final Predicate<Subset> subsetAccessPredicate;
        private final Predicate<FicheMeta> fichePredicate;
        private final Fiches selectedFiches;
        private final ExtractorProvider extractorProvider;
        private final LinkAnalyser linkAnalyser;
        private final MimeTypeResolver mimeTypeResolver;
        private final SyntaxResolver syntaxResolver;

        private InternalExtractionContext(Fichotheque fichotheque, MessageLocalisationProvider messageLocalisationProvider, LangContext langContext, ExtractorProvider extractorProvider, Predicate<Subset> subsetAccessPredicate, Predicate<FicheMeta> fichePredicate, Fiches selectedFiches, LinkAnalyser linkAnalyser, MimeTypeResolver mimeTypeResolver, SyntaxResolver syntaxResolver) {
            this.fichotheque = fichotheque;
            this.messageLocalisationProvider = messageLocalisationProvider;
            this.langContext = langContext;
            this.subsetAccessPredicate = subsetAccessPredicate;
            this.fichePredicate = fichePredicate;
            this.selectedFiches = selectedFiches;
            this.extractorProvider = extractorProvider;
            this.linkAnalyser = linkAnalyser;
            this.mimeTypeResolver = mimeTypeResolver;
            this.syntaxResolver = syntaxResolver;
        }

        @Override
        public Fichotheque getFichotheque() {
            return fichotheque;
        }

        @Override
        public MessageLocalisationProvider getMessageLocalisationProvider() {
            return messageLocalisationProvider;
        }

        @Override
        public LangContext getLangContext() {
            return langContext;
        }

        @Override
        public LinkAnalyser getLinkAnalyser() {
            return linkAnalyser;
        }

        @Override
        public ExtractorProvider getExtractorProvider() {
            return extractorProvider;
        }

        @Override
        public MimeTypeResolver getMimeTypeResolver() {
            return mimeTypeResolver;
        }

        @Override
        public SyntaxResolver getSyntaxResolver() {
            return syntaxResolver;
        }

        @Override
        public Predicate<Subset> getSubsetAccessPredicate() {
            return subsetAccessPredicate;
        }

        @Override
        public Predicate<FicheMeta> getFichePredicate() {
            return fichePredicate;
        }

        @Override
        public Fiches getSelectedFiches() {
            return selectedFiches;
        }

    }

}
