/* FichothequeLib_Tools - Copyright (c) 2013-2022 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.importation.dom;

import java.text.ParseException;
import java.util.function.Consumer;
import net.fichotheque.Fichotheque;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.Subset;
import net.fichotheque.SubsetItem;
import net.fichotheque.SubsetKey;
import net.fichotheque.importation.LiensImport;
import net.fichotheque.include.IncludeKey;
import net.fichotheque.tools.importation.ImportationUtils;
import net.fichotheque.tools.importation.LiensImportBuilder;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.LabelUtils;
import net.mapeadores.util.xml.DOMUtils;
import org.w3c.dom.Element;


/**
 *
 * @author Vincent Calame
 */
public class LiensImportDOMReader {

    private final Fichotheque fichotheque;

    public LiensImportDOMReader(Fichotheque fichotheque) {
        this.fichotheque = fichotheque;
    }

    public boolean readLiensImport(LiensImportBuilder liensImportBuilder, Element element) {
        String tagName = element.getTagName();
        if (tagName.equals("liens")) {
            String type = element.getAttribute("type");
            if (type.equals("replace")) {
                DOMUtils.readChildren(element, new LienConsumer(liensImportBuilder, true));
            } else if (type.equals("append")) {
                DOMUtils.readChildren(element, new LienConsumer(liensImportBuilder, false));
            } else if (type.equals("remove")) {
                DOMUtils.readChildren(element, new IncludeKeyConsumer(liensImportBuilder));
            }
            return true;
        } else {
            return false;
        }
    }


    private class IncludeKeyConsumer implements Consumer<Element> {

        private final LiensImportBuilder liensImportBuilder;

        private IncludeKeyConsumer(LiensImportBuilder liensImportBuilder) {
            this.liensImportBuilder = liensImportBuilder;
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("include-key")) {
                String value = DOMUtils.readSimpleElement(element);
                if (value.equals(FichothequeConstants.LIAGE_NAME)) {
                    liensImportBuilder.setLiageRemoved();
                } else {
                    try {
                        IncludeKey includeKey = IncludeKey.parse(value);
                        liensImportBuilder.addRemovedIncludeKey(includeKey);
                    } catch (ParseException pe) {

                    }
                }
            }
        }

    }


    private class LienConsumer implements Consumer<Element> {

        private final LiensImportBuilder liensImportBuilder;
        private final boolean replace;

        private LienConsumer(LiensImportBuilder liensImportBuilder, boolean replace) {
            this.liensImportBuilder = liensImportBuilder;
            this.replace = replace;
        }

        @Override
        public void accept(Element element) {
            String tagName = element.getTagName();
            if (tagName.equals("lien")) {
                LiensImport.LienImport lienImport = null;
                String origin = element.getAttribute("origin");
                boolean liageOrigin = false;
                IncludeKey originIncludeKey = null;
                if (origin.equals(FichothequeConstants.LIAGE_NAME)) {
                    liageOrigin = true;
                } else {
                    try {
                        originIncludeKey = IncludeKey.parse(origin);
                    } catch (ParseException pe) {
                        return;
                    }
                }
                String mode = element.getAttribute("mode");
                int poids = 1;
                try {
                    poids = Integer.parseInt(element.getAttribute("poids"));
                    if (poids < 1) {
                        poids = 1;
                    }
                } catch (NumberFormatException nfe) {
                }
                SubsetKey subsetKey;
                try {
                    subsetKey = SubsetKey.parse(element.getAttribute("subset"));
                } catch (ParseException pe) {
                    return;
                }
                Subset subset = fichotheque.getSubset(subsetKey);
                if (subset == null) {
                    return;
                }
                String idString = element.getAttribute("id");
                if (idString.length() > 0) {
                    try {
                        int id = Integer.parseInt(idString);
                        SubsetItem subsetItem = subset.getSubsetItemById(id);
                        if (subsetItem != null) {
                            lienImport = ImportationUtils.toLienImport(liageOrigin, originIncludeKey, mode, subsetItem, poids);
                        }
                    } catch (NumberFormatException nfe) {
                    }
                } else {
                    try {
                        Lang lang = Lang.parse(element.getAttribute("lang"));
                        CleanedString labelString = CleanedString.newInstance(element.getAttribute("label"));
                        if (labelString != null) {
                            lienImport = ImportationUtils.toLienImport(liageOrigin, originIncludeKey, mode, subset, LabelUtils.toLabel(lang, labelString), poids);
                        }
                    } catch (ParseException lie) {
                    }
                }
                if (lienImport != null) {
                    if (replace) {
                        liensImportBuilder.addReplaceLienImport(lienImport);
                    } else {
                        liensImportBuilder.addAppendLienImport(lienImport);
                    }
                }
            }
        }

    }

}
